#include <stdio.h>
#include<stdlib.h>
#include<math.h>
#include"SimpleImage.h"
#include"pgmio.h"
#include"ppmio.h"
#include"color.h"
#include"otsu.h"
#include"label.h"


double getVal0to255(double val){
  int vali =((int)(((double)(255.0*val))));
  if(fabs((double)(val-vali))>=0.5)vali++;
  if(vali<0)vali=0;
  if(vali>255)vali=255;
  return ((double)(vali));
}


int main(int argc, char* argv[])
{
  Image *in = new Image();

  getPGM(in,argv[1]);
  
  Image *out = new Image(in->sx,in->sy);
  
  /* main processing */
  int i,j;

  int HISTSIZE=256;
  Otsu *myotsu = new Otsu(HISTSIZE);

  double threshold = ((double)(myotsu->find_otsu_threshold(in)));
  printf("Otsu's threshold value = %lf\n",threshold);
  delete myotsu;

  for(i=0;i<in->sy;i++)
    for(j=0;j<in->sx;j++){
      if(in->img[i][j]>=threshold){
	out->img[i][j] = 255.0;
      }else{
	out->img[i][j] = 0.0;
      }
    }
  
  //Labeling
  Labeling *mylabel = new Labeling();
  int labeledN = mylabel->doLabeling(out);
  out->gray = labeledN;
  printf("Number of connected objects = %d\n",labeledN);
  //End Labeling
  delete mylabel;



  //Cut small segments.
  int *hist = new int[labeledN]; 
  
  for(i=0;i<labeledN;i++){
    hist[i]=0;
  }
  for(i=0;i<in->sy;i++)
    for(j=0;j<in->sx;j++){
      int val = ((int)(out->img[i][j]));
      if(val<0)val=0;
      if(val>=labeledN)val = labeledN-1;
      hist[val]++;
    }
  
  int cutthreshold = atoi(argv[3]);
  
  int *map = new int[labeledN];
  int baseID=0;
  for(i=0;i<labeledN;i++){
    if(hist[i]<=cutthreshold){
      map[i]=-1;
    }else{
      map[i]=baseID;
      baseID++;
    }
  }
  
  
  for(i=0;i<in->sy;i++)
    for(j=0;j<in->sx;j++){
      int val = ((int)(out->img[i][j]));
      if(val<0)val=0;
      if(val>=labeledN)val = labeledN-1;
      
      if(map[val]==-1){
	out->img[i][j] = -1.0;
      }else{
	out->img[i][j] = ((double)(map[val]));
	
      }
      
    }
  baseID++;
  printf("Number of new segments = %d\n",baseID);
  out->gray=baseID;
  
  
  delete [] map;
  delete [] hist;
  //for Coloring
  double maxval=out->img[0][0];
  for(i=0;i<out->sy;i++)
    for(j=0;j<out->sx;j++){
      if(maxval<out->img[i][j]){
	maxval = out->img[i][j];
      }
    }
  if(maxval<=0.0)maxval=1.0;
  
  Image *outR = new Image(in->sx,in->sy);
  Image *outG = new Image(in->sx,in->sy);
  Image *outB = new Image(in->sx,in->sy);
  
  for(i=0;i<in->sy;i++)
    for(j=0;j<in->sx;j++){
      if(out->img[i][j] ==-1.0){
	outR->img[i][j] = 0.0;
	outG->img[i][j] = 0.0;
	outB->img[i][j] = 0.0;
      }else{
	double val = biR((out->img[i][j]/maxval));
	outR->img[i][j] = getVal0to255(val);
	val = biG((out->img[i][j]/maxval));
	outG->img[i][j] = getVal0to255(val);
	val = biB((out->img[i][j]/maxval));
	outB->img[i][j] = getVal0to255(val);
      }
      
    }
  

  

  /* end main processing */

  
  savePPM(outR,outG,outB,argv[2]);


  delete outR;
  delete outG;
  delete outB;
  delete out;
  delete in;
  
  return 0;
}
